# HashMap\[K: [Any](builtin-Any) #share, V: [Any](builtin-Any) #share, H: [HashFunction](collections-HashFunction)\[K\] val\]

A persistent map based on the Compressed Hash Array Mapped Prefix-tree from
'Optimizing Hash-Array Mapped Tries for Fast and Lean Immutable JVM
Collections' by Michael J. Steindorfer and Jurgen J. Vinju

## Usage
```pony
use "collections/persistent"

actor Main
  new create(env: Env) =>
    try
      let m1 = Map[String, U32] // {}
      // Update returns a new map with the provided key set
      // to the provided value. The old map is unchanged.
      let m2 = m1("a") = 5 // {a: 5}
      let m3 = m2("b") = 10 // {a: 5, b: 10}
      let m4 = m3.remove("a")? // {b: 10}
      // You can create a new map from key value pairs.
      let m5 = Map[String, U32].concat([("a", 2); ("b", 3)].values()) // {a: 2, b: 3}
    end
```


```pony
class val HashMap[K: Any #share, V: Any #share, H: HashFunction[K] val]
```

## Constructors

### create

```pony
new val create()
: HashMap[K, V, H] val^
```

#### Returns

* [HashMap](collections-persistent-HashMap)\[K, V, H\] val^

---

### _create

```pony
new val _create(
  r: _MapNode[K, V, H] val,
  s: USize val)
: HashMap[K, V, H] val^
```
#### Parameters

*   r: [_MapNode](collections-persistent-_MapNode)\[K, V, H\] val
*   s: [USize](builtin-USize) val

#### Returns

* [HashMap](collections-persistent-HashMap)\[K, V, H\] val^

---

## Public Functions

### apply

Attempt to get the value corresponding to k.


```pony
fun val apply(
  k: K)
: V ?
```
#### Parameters

*   k: K

#### Returns

* V ?

---

### size

Return the amount of key-value pairs in the Map.


```pony
fun val size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### update

Update the value associated with the provided key.


```pony
fun val update(
  key: K,
  value: V)
: HashMap[K, V, H] val
```
#### Parameters

*   key: K
*   value: V

#### Returns

* [HashMap](collections-persistent-HashMap)\[K, V, H\] val

---

### remove

Try to remove the provided key from the Map.


```pony
fun val remove(
  k: K)
: HashMap[K, V, H] val ?
```
#### Parameters

*   k: K

#### Returns

* [HashMap](collections-persistent-HashMap)\[K, V, H\] val ?

---

### get_or_else

Get the value associated with provided key if present. Otherwise,
return the provided alternate value.


```pony
fun val get_or_else(
  k: K,
  alt: V)
: V
```
#### Parameters

*   k: K
*   alt: V

#### Returns

* V

---

### contains

Check whether the node contains the provided key.


```pony
fun val contains(
  k: K)
: Bool val
```
#### Parameters

*   k: K

#### Returns

* [Bool](builtin-Bool) val

---

### concat

Add the K, V pairs from the given iterator to the map.


```pony
fun val concat(
  iter: Iterator[(K , V)] ref)
: HashMap[K, V, H] val
```
#### Parameters

*   iter: [Iterator](builtin-Iterator)\[(K , V)\] ref

#### Returns

* [HashMap](collections-persistent-HashMap)\[K, V, H\] val

---

### keys

```pony
fun val keys()
: MapKeys[K, V, H] ref
```

#### Returns

* [MapKeys](collections-persistent-MapKeys)\[K, V, H\] ref

---

### values

```pony
fun val values()
: MapValues[K, V, H] ref
```

#### Returns

* [MapValues](collections-persistent-MapValues)\[K, V, H\] ref

---

### pairs

```pony
fun val pairs()
: MapPairs[K, V, H] ref
```

#### Returns

* [MapPairs](collections-persistent-MapPairs)\[K, V, H\] ref

---

## Private Functions

### _root_node

```pony
fun box _root_node()
: _MapNode[K, V, H] val
```

#### Returns

* [_MapNode](collections-persistent-_MapNode)\[K, V, H\] val

---

