# Nil\[A: A\]

The empty list of As.


```pony
primitive val Nil[A: A] is
  ReadSeq[val->A] box
```

#### Implements

* [ReadSeq](builtin-ReadSeq)\[val->A\] box

---

## Constructors

### create

```pony
new val create()
: Nil[A] val^
```

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val^

---

## Public Functions

### size

Returns the size of the list.


```pony
fun box size()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### apply

Returns the i-th element of the sequence. For the empty list this call will
always error because any index will be out of bounds.


```pony
fun box apply(
  i: USize val)
: val->A ?
```
#### Parameters

*   i: [USize](builtin-USize) val

#### Returns

* val->A ?

---

### values

Returns an empty iterator over the elements of the empty list.


```pony
fun box values()
: Iterator[val->A] ref^
```

#### Returns

* [Iterator](builtin-Iterator)\[val->A\] ref^

---

### is_empty

Returns a Bool indicating if the list is empty.


```pony
fun box is_empty()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### is_non_empty

Returns a Bool indicating if the list is non-empty.


```pony
fun box is_non_empty()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---

### head

Returns an error, since Nil has no head.


```pony
fun box head()
: val->A ?
```

#### Returns

* val->A ?

---

### tail

Returns an error, since Nil has no tail.


```pony
fun box tail()
: (Cons[A] val | Nil[A] val) ?
```

#### Returns

* ([Cons](collections-persistent-Cons)\[A\] val | [Nil](collections-persistent-Nil)\[A\] val) ?

---

### reverse

The reverse of the empty list is the empty list.


```pony
fun box reverse()
: Nil[A] val
```

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val

---

### prepend

Builds a new list with an element added to the front of this list.


```pony
fun box prepend(
  a: val->A!)
: Cons[A] val
```
#### Parameters

*   a: val->A!

#### Returns

* [Cons](collections-persistent-Cons)\[A\] val

---

### concat

The concatenation of any list l with the empty list is l.


```pony
fun box concat(
  l: (Cons[A] val | Nil[A] val))
: (Cons[A] val | Nil[A] val)
```
#### Parameters

*   l: ([Cons](collections-persistent-Cons)\[A\] val | [Nil](collections-persistent-Nil)\[A\] val)

#### Returns

* ([Cons](collections-persistent-Cons)\[A\] val | [Nil](collections-persistent-Nil)\[A\] val)

---

### map\[B: B\]

Mapping a function from A to B over the empty list yields the
empty list of Bs.


```pony
fun box map[B: B](
  f: {(val->A): val->B}[A, B] box)
: Nil[B] val
```
#### Parameters

*   f: {(val->A): val->B}[A, B] box

#### Returns

* [Nil](collections-persistent-Nil)\[B\] val

---

### flat_map\[B: B\]

Flatmapping a function from A to B over the empty list yields the
empty list of Bs.


```pony
fun box flat_map[B: B](
  f: {(val->A): List[B]}[A, B] box)
: Nil[B] val
```
#### Parameters

*   f: {(val->A): List[B]}[A, B] box

#### Returns

* [Nil](collections-persistent-Nil)\[B\] val

---

### for_each

Applying a function to every member of the empty list is a no-op.


```pony
fun box for_each(
  f: {(val->A)}[A] box)
: None val
```
#### Parameters

*   f: {(val->A)}[A] box

#### Returns

* [None](builtin-None) val

---

### filter

Filtering the empty list yields the empty list.


```pony
fun box filter(
  f: {(val->A): Bool}[A] box)
: Nil[A] val
```
#### Parameters

*   f: {(val->A): Bool}[A] box

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val

---

### fold\[B: B\]

Folding over the empty list yields the initial accumulator.


```pony
fun box fold[B: B](
  f: {(B, val->A): B^}[A, B] box,
  acc: B)
: B
```
#### Parameters

*   f: {(B, val->A): B^}[A, B] box
*   acc: B

#### Returns

* B

---

### every

Any predicate is true of every member of the empty list.


```pony
fun box every(
  f: {(val->A): Bool}[A] box)
: Bool val
```
#### Parameters

*   f: {(val->A): Bool}[A] box

#### Returns

* [Bool](builtin-Bool) val

---

### exists

For any predicate, there is no element that satisfies it in the empty
list.


```pony
fun box exists(
  f: {(val->A): Bool}[A] box)
: Bool val
```
#### Parameters

*   f: {(val->A): Bool}[A] box

#### Returns

* [Bool](builtin-Bool) val

---

### partition

The only partition of the empty list is two empty lists.


```pony
fun box partition(
  f: {(val->A): Bool}[A] box)
: (Nil[A] val , Nil[A] val)
```
#### Parameters

*   f: {(val->A): Bool}[A] box

#### Returns

* ([Nil](collections-persistent-Nil)\[A\] val , [Nil](collections-persistent-Nil)\[A\] val)

---

### drop

There are no elements to drop from the empty list.


```pony
fun box drop(
  n: USize val)
: Nil[A] val
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val

---

### drop_while

There are no elements to drop from the empty list.


```pony
fun box drop_while(
  f: {(val->A): Bool}[A] box)
: Nil[A] val
```
#### Parameters

*   f: {(val->A): Bool}[A] box

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val

---

### take

There are no elements to take from the empty list.


```pony
fun box take(
  n: USize val)
: Nil[A] val
```
#### Parameters

*   n: [USize](builtin-USize) val

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val

---

### take_while

There are no elements to take from the empty list.


```pony
fun box take_while(
  f: {(val->A): Bool}[A] box)
: Nil[A] val
```
#### Parameters

*   f: {(val->A): Bool}[A] box

#### Returns

* [Nil](collections-persistent-Nil)\[A\] val

---

### contains\[optional T: (A & [HasEq](builtin-HasEq)\[A!\] #read)\]

```pony
fun val contains[optional T: (A & HasEq[A!] #read)](
  a: val->T)
: Bool val
```
#### Parameters

*   a: val->T

#### Returns

* [Bool](builtin-Bool) val

---

### eq

```pony
fun box eq(
  that: Nil[A] val)
: Bool val
```
#### Parameters

*   that: [Nil](collections-persistent-Nil)\[A\] val

#### Returns

* [Bool](builtin-Bool) val

---

### ne

```pony
fun box ne(
  that: Nil[A] val)
: Bool val
```
#### Parameters

*   that: [Nil](collections-persistent-Nil)\[A\] val

#### Returns

* [Bool](builtin-Bool) val

---

