# Match

Contains match data for a combination of a regex and a subject.


```pony
class ref Match
```

## Constructors

### _create

Store a match, a subject, and a size.


```pony
new ref _create(
  subject: (String val | Array[U8 val] val),
  m: Pointer[_Match val] ref)
: Match ref^
```
#### Parameters

*   subject: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)
*   m: [Pointer](builtin-Pointer)\[[_Match](regex-_Match) val\] ref

#### Returns

* [Match](regex-Match) ref^

---

## Public Functions

### size

Returns the capture size of the match.


```pony
fun box size()
: U32 val
```

#### Returns

* [U32](builtin-U32) val

---

### start_pos

Returns the character position of the first character in the match.


```pony
fun box start_pos()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### end_pos

Returns the character position of the last character in the match.


```pony
fun box end_pos()
: USize val
```

#### Returns

* [USize](builtin-USize) val

---

### apply\[optional A: (([String](builtin-String) iso | [Array](builtin-Array)\[[U8](builtin-U8) val\] iso) & [Seq](builtin-Seq)\[[U8](builtin-U8) val\] iso)\]

Returns a capture by number. Raises an error if the index is out of bounds.


```pony
fun box apply[optional A: ((String iso | Array[U8 val] iso) & Seq[U8 val] iso)](
  i: U32 val)
: A^ ?
```
#### Parameters

*   i: [U32](builtin-U32) val

#### Returns

* A^ ?

---

### find\[optional A: (([String](builtin-String) iso | [Array](builtin-Array)\[[U8](builtin-U8) val\] iso) & [Seq](builtin-Seq)\[[U8](builtin-U8) val\] iso)\]

Returns a capture by name. Raises an error if the named capture does not
exist.


```pony
fun box find[optional A: ((String iso | Array[U8 val] iso) & Seq[U8 val] iso)](
  name: String box)
: A^ ?
```
#### Parameters

*   name: [String](builtin-String) box

#### Returns

* A^ ?

---

### groups

Returns all of the captured subgroups. Groups that failed to capture
anything will contain the empty string.


```pony
fun box groups()
: Array[String val] iso^
```

#### Returns

* [Array](builtin-Array)\[[String](builtin-String) val\] iso^

---

### dispose

Free the underlying PCRE2 data.


```pony
fun ref dispose()
: None val
```

#### Returns

* [None](builtin-None) val

---

## Private Functions

### _final

Free the underlying PCRE2 data.


```pony
fun box _final()
: None val
```

#### Returns

* [None](builtin-None) val

---

