# Readline

Line editing, history, and tab completion.


```pony
class ref Readline is
  ANSINotify ref
```

#### Implements

* [ANSINotify](term-ANSINotify) ref

---

## Constructors

### create

```pony
new iso create(
  notify: ReadlineNotify iso,
  out: OutStream tag,
  path: (FilePath val | None val) = seq,
  maxlen: USize val = seq)
: Readline iso^
```
#### Parameters

*   notify: [ReadlineNotify](term-ReadlineNotify) iso
*   out: [OutStream](builtin-OutStream) tag
*   path: ([FilePath](files-FilePath) val | [None](builtin-None) val) = seq
*   maxlen: [USize](builtin-USize) val = seq

#### Returns

* [Readline](term-Readline) iso^

---

## Public Functions

### apply

Receives input.


```pony
fun ref apply(
  term: ANSITerm ref,
  input: U8 val)
: None val
```
#### Parameters

*   term: [ANSITerm](term-ANSITerm) ref
*   input: [U8](builtin-U8) val

#### Returns

* [None](builtin-None) val

---

### prompt

Set a new prompt, unblock, and handle the pending queue.


```pony
fun ref prompt(
  term: ANSITerm ref,
  value: String val)
: None val
```
#### Parameters

*   term: [ANSITerm](term-ANSITerm) ref
*   value: [String](builtin-String) val

#### Returns

* [None](builtin-None) val

---

### closed

No more input is available.


```pony
fun ref closed()
: None val
```

#### Returns

* [None](builtin-None) val

---

### up

Previous line.


```pony
fun ref up(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### down

Next line.


```pony
fun ref down(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### left

Move left.


```pony
fun ref left(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### right

Move right.


```pony
fun ref right(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### home

Beginning of the line.


```pony
fun ref home(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### end_key

End of the line.


```pony
fun ref end_key(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### delete

Forward delete.


```pony
fun ref delete(
  ctrl: Bool val = seq,
  alt: Bool val = seq,
  shift: Bool val = seq)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val = seq
*   alt: [Bool](builtin-Bool) val = seq
*   shift: [Bool](builtin-Bool) val = seq

#### Returns

* [None](builtin-None) val

---

### insert

```pony
fun ref insert(
  ctrl: Bool val,
  alt: Bool val,
  shift: Bool val)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val
*   alt: [Bool](builtin-Bool) val
*   shift: [Bool](builtin-Bool) val

#### Returns

* [None](builtin-None) val

---

### page_up

```pony
fun ref page_up(
  ctrl: Bool val,
  alt: Bool val,
  shift: Bool val)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val
*   alt: [Bool](builtin-Bool) val
*   shift: [Bool](builtin-Bool) val

#### Returns

* [None](builtin-None) val

---

### page_down

```pony
fun ref page_down(
  ctrl: Bool val,
  alt: Bool val,
  shift: Bool val)
: None val
```
#### Parameters

*   ctrl: [Bool](builtin-Bool) val
*   alt: [Bool](builtin-Bool) val
*   shift: [Bool](builtin-Bool) val

#### Returns

* [None](builtin-None) val

---

### fn_key

```pony
fun ref fn_key(
  i: U8 val,
  ctrl: Bool val,
  alt: Bool val,
  shift: Bool val)
: None val
```
#### Parameters

*   i: [U8](builtin-U8) val
*   ctrl: [Bool](builtin-Bool) val
*   alt: [Bool](builtin-Bool) val
*   shift: [Bool](builtin-Bool) val

#### Returns

* [None](builtin-None) val

---

### size

```pony
fun ref size(
  rows: U16 val,
  cols: U16 val)
: None val
```
#### Parameters

*   rows: [U16](builtin-U16) val
*   cols: [U16](builtin-U16) val

#### Returns

* [None](builtin-None) val

---

## Private Functions

### _backspace

Backward delete.


```pony
fun ref _backspace()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _clear

Clear the screen.


```pony
fun ref _clear()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _swap

Swap the previous character with the current one.


```pony
fun ref _swap()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _delete_prev_word

Delete the previous word.


```pony
fun ref _delete_prev_word()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _tab

Tab completion.

TODO: Improve this.


```pony
fun ref _tab()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _dispatch

Send a finished line to the notifier.


```pony
fun ref _dispatch(
  term: ANSITerm tag)
: None val
```
#### Parameters

*   term: [ANSITerm](term-ANSITerm) tag

#### Returns

* [None](builtin-None) val

---

### _handle_line

Dispatch a single line.


```pony
fun ref _handle_line(
  term: ANSITerm tag,
  line: String val)
: None val
```
#### Parameters

*   term: [ANSITerm](term-ANSITerm) tag
*   line: [String](builtin-String) val

#### Returns

* [None](builtin-None) val

---

### _refresh_line

Refresh the line on screen.


```pony
fun ref _refresh_line()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _add_history

Add a line to the history, trimming an earlier line if necessary.


```pony
fun ref _add_history(
  line: String val)
: None val
```
#### Parameters

*   line: [String](builtin-String) val

#### Returns

* [None](builtin-None) val

---

### _load_history

Load the history from a file.


```pony
fun ref _load_history()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _save_history

Write the history back to a file.


```pony
fun box _save_history()
: None val
```

#### Returns

* [None](builtin-None) val

---

